using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data;
using System.Runtime.Serialization;

using CMS.CMSEcommerce;
using CMS.IECommerce;

namespace CMS.CustomECommerceProvider 
{
    /// <summary>
    /// Custom shopping cart info provider
    /// </summary>
    public class CustomShoppingCartInfoProvider: IShoppingCartInfoProvider
    {
        /// <summary>
        /// Evaluates shopping cart content - calculates all taxes, discounts and subtotals.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart object with all neccessary data for calculation.</param>       
        public void EvaluateShoppingCartContent(object shoppingCart)
        {
            ShoppingCartInfoProvider.EvaluateShoppingCartContent((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }


        /// <summary>
        /// Evaluates shopping cart content - calculates all taxes, discounts and subtotals.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart object with all neccessary data for calculation.</param>
        /// <param name="evaluateForInvoice">Indicates if shopping cart content is evaluated for invoice.</param>
        public void EvaluateShoppingCartContent(object shoppingCart, bool evaluateForInvoice)
        {
            //ShoppingCartInfoProvider.EvaluateShoppingCartContent((CMS.Ecommerce.ShoppingCartInfo)shoppingCart, evaluateForInvoice);
            CMS.Ecommerce.ShoppingCartInfo cart = shoppingCart as CMS.Ecommerce.ShoppingCartInfo;
            if (cart != null)
            {
                ShoppingCartInfoProvider.EvaluateShoppingCartContent(cart, evaluateForInvoice);
                
                CMS.Ecommerce.TaxClassInfo tci = TaxClassInfoProvider.GetTaxClassInfo("ShippingTax");

                if (cart.ShoppingCartCustomData["ShippingTax"] != null)
                {
                    DataRow dr = cart.ShoppingCartTaxTable.NewRow();
                    dr["TaxClassID"] = tci.TaxClassID;
                    dr["TaxClassDisplayName"] = tci.TaxClassDisplayName;

                    // Get the added tax value from cart custom data hashtable
                    double taxValue = double.Parse(cart.ShoppingCartCustomData["ShippingTax"]);
                    
                    // Add the shipping tax summary into the tax table
                    CMS.Ecommerce.ShoppingCartInfo.AddTaxSummary(cart.ShoppingCartTaxTable, dr, taxValue);
                }
            }
        }


        /// <summary>
        /// Deletes specified shoppingCart.
        /// </summary>
        /// <param name="shoppingCart">ShoppingCart object.</param>
        public void DeleteShoppingCartInfo(object shoppingCart)
        {
            ShoppingCartInfoProvider.DeleteShoppingCartInfo((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }


        /// <summary>
        /// Returns the ShoppingCartInfo structure for the specified shoppingCart.
        /// </summary>
        /// <param name="shoppingCartId">ShoppingCart id.</param>
        public object GetShoppingCartInfo(int shoppingCartId)
        {
            return ShoppingCartInfoProvider.GetShoppingCartInfo(shoppingCartId);
        }


        /// <summary>
        /// Returns the ShoppingCartInfo structure for the specified shoppingCart.
        /// </summary>
        /// <param name="shoppingCartGUID">ShoppingCart GUID.</param>
        public object GetShoppingCartInfo(Guid shoppingCartGUID)
        {
            return ShoppingCartInfoProvider.GetShoppingCartInfo(shoppingCartGUID);
        }


        /// <summary>
        /// Returns the ShoppingCartInfo structure for the specified user and site.
        /// </summary>
        /// <param name="userId">User Id.</param>
        /// <param name="siteName">Site name.</param>
        public object GetShoppingCartInfo(int userId, string siteName)
        {
            return ShoppingCartInfoProvider.GetShoppingCartInfo(userId, siteName);
        }


        /// <summary>
        /// Returns filled ShoppingCartInfo object from the specified order data.
        /// </summary>
        /// <param name="orderId">Order ID.</param>
        public object GetShoppingCartInfoFromOrder(int orderId)
        {
            return ShoppingCartInfoProvider.GetShoppingCartInfoFromOrder(orderId);
        }


        /// <summary>
        /// Sets (updates or inserts) specified shoppingCart.
        /// </summary>
        /// <param name="shoppingCart">ShoppingCart to set.</param>
        public void SetShoppingCartInfo(object shoppingCart)
        {
            ShoppingCartInfoProvider.SetShoppingCartInfo((CMS.Ecommerce.ShoppingCartInfo) shoppingCart);
        }


        /// <summary>
        /// Deletes specified shoppingCart.
        /// </summary>
        /// <param name="shoppingCartId">ShoppingCart id.</param>
        public void DeleteShoppingCartInfo(int shoppingCartId)
        {
            ShoppingCartInfoProvider.DeleteShoppingCartInfo(shoppingCartId);
        }


        /// <summary>
        /// Deletes specified shopping cart.
        /// </summary>
        /// <param name="shoppingCartGuid">ShoppingCart GUID.</param>
        public void DeleteShoppingCartInfo(Guid shoppingCartGuid)
        {
            ShoppingCartInfoProvider.DeleteShoppingCartInfo(shoppingCartGuid);
        }


        /// <summary>
        /// Deletes specified user's Shopping cart.
        /// </summary>
        /// <param name="userId">User id.</param>
        /// <param name="siteName">Site name.</param>
        public void DeleteShoppingCartInfo(int userId, string siteName)
        {
            ShoppingCartInfoProvider.DeleteShoppingCartInfo(userId, siteName);
        }


        /// <summary>
        /// Deletes all shopping cart items.
        /// </summary>
        /// <param name="cartId">Shopping cart ID.</param>
        public void DeleteShoppingCartItems(int cartId)
        {
            ShoppingCartInfoProvider.DeleteShoppingCartItems(cartId);
        }  


        /// <summary>
        /// Empty shopping cart.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart to empty.</param>
        public void EmptyShoppingCart(object shoppingCart)
        {
            ShoppingCartInfoProvider.EmptyShoppingCart((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }


        /// <summary>
        /// Returns shopping carts that are older than 30 days from now.
        /// </summary>     
        public DataSet GetExpiredShoppingCarts()
        {
            return ShoppingCartInfoProvider.GetExpiredShoppingCarts();
        }


        /// <summary>
        /// Returns new empty ShoppingCartInfo object.
        /// </summary>
        /// <param name="siteId">Site ID.</param>
        public object CreateShoppingCartInfo(int siteId)
        {
            return CMS.Ecommerce.ShoppingCartInfo.CreateShoppingCartInfo(siteId);         
        }


        /// <summary>
        /// Returns new ShoppingCartInfo object created from the given DataRow.
        /// </summary>
        /// <param name="dr">DataRow with shopping cart data.</param>
        public object CreateShoppingCartInfo(DataRow dr)
        {
            return CMS.Ecommerce.ShoppingCartInfo.CreateShoppingCartInfo(dr);
        }


        /// <summary>
        /// Returns new ShoppingCartInfo object created from deserialized input data.
        /// </summary>
        /// <param name="info">Data for deserialization the ShoppingCartInfo object should be created from.</param>
        /// <param name="context">Streaming context.</param>
        public object CreateShoppingCartInfo(SerializationInfo info, StreamingContext context)
        {
            return CMS.Ecommerce.ShoppingCartInfo.CreateShoppingCartInfo(info, context);           
        }


        /// <summary>
        /// Clears customer's private data from shopping cart.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart to clear private data from.</param>
        public void ClearShoppingCartPrivateData(object shoppingCart)
        {
            ShoppingCartInfoProvider.ClearShoppingCartPrivateData((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }
        

        /// <summary>
        /// Actualizes products' available items according to the products' units of the specified shopping cart.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart with some products.</param>
        public void ActualizeSKUAvailableItems(object shoppingCart)
        {
            ShoppingCartInfoProvider.ActualizeSKUAvailableItems((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }


        /// <summary>
        /// Inserts or updates order according to the shopping cart data.
        /// </summary>
        /// <param name="shoppingCart">Shopping cart with order data.</param>
        public void SetOrder(object shoppingCart)
        {
            ShoppingCartInfoProvider.SetOrder((CMS.Ecommerce.ShoppingCartInfo)shoppingCart);
        }
    }
}
